package com.qmth.boot.ai.api.demo.test;

import com.fasterxml.jackson.databind.json.JsonMapper;
import com.qmth.boot.core.ai.client.DocApiClient;
import com.qmth.boot.core.ai.client.LlmApiClient;
import com.qmth.boot.core.ai.client.OcrApiClient;
import com.qmth.boot.core.ai.model.doc.ParseDocTaskResult;
import com.qmth.boot.core.ai.model.llm.*;
import com.qmth.boot.core.ai.model.llm.endpoint.ChatEndpoint;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreModel;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreRequest;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreResult;
import com.qmth.boot.core.ai.model.ocr.OcrType;
import com.qmth.boot.core.ai.service.AiService;
import com.qmth.boot.core.retrofit.utils.SignatureInfo;
import com.qmth.boot.core.retrofit.utils.UploadFile;
import okhttp3.*;
import okio.BufferedSource;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@RunWith(SpringRunner.class)
@SpringBootTest
public class AiApiDemoTest {

    @Resource
    private AiService aiService;

    @Resource
    private LlmApiClient llmApiClient;

    @Resource
    private OcrApiClient ocrApiClient;

    @Resource
    private DocApiClient docApiClient;

    private SignatureInfo secret;

    private String baseUrl;

    @Before
    public void init() {
        // baseUrl = "https://solar.qmth.com.cn";
        // baseUrl = "http://192.168.10.83:8090";
        // baseUrl = "http://localhost:8090";

        String accessKey = "41a9c68b71c9486db62de90980bd7e9a";
        String accessSecret = "yhe22bNXMRljatkvJA4f56by9dIAJBjv";
        secret = SignatureInfo.secret(accessKey, accessSecret);
    }

    @Test
    public void testAll() throws Exception {
        // testOcrImage();
        // testParseDoc();
        // autoScore();
        // autoGenerateQuestionWithStream();
        // autoGenerateQuestion();
        // chat();
        // getPromptTemplate();
        // getLlmBalance();
    }

    private void testOcrImage() throws Exception {
        File file = new File("D:\\home\\test.png");
        MultipartBody.Part part = UploadFile.build("image", file.getName(), file);

        String result = ocrApiClient.forImage(baseUrl, secret, OcrType.HANDWRITING, part);
        System.out.println(result);
    }

    private void testParseDoc() throws Exception {
        File file = new File("D:\\home\\大纲.pdf");
        MultipartBody.Part part = UploadFile.build("file", file.getName(), file);

        // ParseDocTask parseDocTask = docApiClient.parseDocTask(baseUrl, secret, part);
        // System.out.println(parseDocTask.getTaskId());

        // String taskId = parseDocTask.getTaskId();
        String taskId = "M0BAYmFpZHUuZG9jLkJhaWR1UGFyc2VEb2NDbGllbnRAQHRhc2stdnVrV1RHZnNSbFNJdDFBaDNsRUxqNzRRczJzdXNhOXk=";
        ParseDocTaskResult parseDocTaskResult = docApiClient.parseDocTaskQuery(baseUrl, secret, taskId);
        System.out.println(parseDocTaskResult.getStatus());
        System.out.println(parseDocTaskResult.getContent());
    }

    private void autoScore() throws Exception {
        AutoScoreRequest request = new AutoScoreRequest();
        request.setSubjectName("语文");
        request.setQuestionBody("请列举两首李白的诗词？");
        request.setTotalScore(100.0);
        request.setIntervalScore(0.5);
        // request.setStudentAnswer("《静夜思》");
        request.setStudentAnswer("《静夜思》《望庐山瀑1布》");

        request.setScoreModel(AutoScoreModel.POINT);
        request.addStandardAnswer("《静夜思》", 50);
        request.addStandardAnswer("《望庐山瀑布》", 50);

        // request.setScoreModel(AutoScoreModel.LEVEL);
        // request.addStandardAnswer("《静夜思》", 0, 50);
        // request.addStandardAnswer("《望庐山瀑布》", 0, 50);
        // request.addStandardAnswer("《静夜思》《望庐山瀑布》", 51, 100);

        AutoScoreResult result = aiService.autoScore(baseUrl, secret, request);
        System.out.println(new JsonMapper().writeValueAsString(result));
    }

    private void autoGenerateQuestionWithStream() throws Exception {
        Map<String, Object> param = this.buildQuestionParam();
        ChatEndpoint endpoint = llmApiClient.chatEndpoint(baseUrl, secret, LlmAppType.AUTO_GENERATE_QUESTION, param, true, true);
        System.out.println(new JsonMapper().writeValueAsString(endpoint));

        Request.Builder requestBuilder = new Request.Builder();
        // requestBuilder.url("https://dashscope.aliyuncs.com/compatible-mode/v1/chat/completions");
        requestBuilder.url(endpoint.getUrl());
        for (Map.Entry<String, String> entry : endpoint.getHeaders().entrySet()) {
            requestBuilder.addHeader(entry.getKey(), entry.getValue());
        }
        RequestBody requestBody = RequestBody.create(MediaType.get("application/json"), endpoint.getRequestBody());
        Request request = requestBuilder.post(requestBody).build();

        OkHttpClient httpClient = new OkHttpClient.Builder()
                .readTimeout(60, TimeUnit.SECONDS)
                .connectTimeout(60, TimeUnit.SECONDS)
                .build();
        httpClient.newCall(request).enqueue(new Callback() {
            @Override
            public void onResponse(Call call, Response response) throws IOException {
                if (!response.isSuccessful()) {
                    System.err.println("接口响应错误！" + response);
                    return;
                }

                try (ResponseBody body = response.body()) {
                    BufferedSource source = body.source();

                    while (!source.exhausted()) {
                        String line = source.readUtf8Line();
                        if (line == null || line.isEmpty()) {
                            continue;
                        }
                        // 流式输出Chunk内容
                        System.out.println(line);
                    }

                    System.out.println("解析内容完成！");
                } catch (Exception e) {
                    System.err.println("解析内容错误！" + e.getMessage());
                }
            }

            @Override
            public void onFailure(Call call, IOException e) {
                System.err.println("接口调用失败！" + e.getMessage());
            }
        });

        TimeUnit.SECONDS.sleep(30);
    }

    private void autoGenerateQuestion() throws Exception {
        Map<String, Object> param = this.buildQuestionParam();
        List<String> result = aiService.autoGenerateQuestion(baseUrl, secret, param);
        System.out.println(new JsonMapper().writeValueAsString(result));
    }

    private Map<String, Object> buildQuestionParam() {
        return new HashMap<String, Object>() {{
            put("courseName", "语文");
            put("questionTypeName", "多选题");
            put("syllabus", "熟练掌握诗词");// 教学大纲内容
            put("syllabusNotes", "");// 教学大纲补充说明
            put("knowledgePoint", "诗词");// 知识点
            put("knowledgeNotes", "作者出处");// 知识点补充说明
            // SINGLE_ANSWER_QUESTION、MULTIPLE_ANSWER_QUESTION、BOOL_ANSWER_QUESTION、FILL_BLANK_QUESTION、TEXT_ANSWER_QUESTION
            put("questionType", "MULTIPLE_ANSWER_QUESTION");
            // optionCount、blankCount、answerCount
            put("optionCount", 4);
        }};
    }

    private void chat() throws Exception {
        ChatRequest request = new ChatRequest();
        request.addMessage(ChatRole.user, "请输出1加1等于几？");

        ChatResult result = llmApiClient.chat(baseUrl, secret, LlmAppType.AUTO_SCORE, request);
        List<String> values = result.getChoices().stream().filter(choice -> choice.getMessage().getRole() == ChatRole.assistant)
                .map(choice -> choice.getMessage().getContent()).collect(Collectors.toList());
        System.out.println(new JsonMapper().writeValueAsString(values));
    }

    private void getPromptTemplate() throws Exception {
        PromptTemplate result = llmApiClient.getPromptTemplate(baseUrl, secret, LlmAppType.AUTO_SCORE);
        System.out.println(new JsonMapper().writeValueAsString(result));
    }

    private void getLlmBalance() throws Exception {
        // LlmAppBalance result = llmApiClient.consumeBalance(baseUrl, secret, LlmAppType.AUTO_SCORE);
        LlmAppBalance result = aiService.getLlmBalance(baseUrl, secret, LlmAppType.AUTO_SCORE);
        System.out.println(new JsonMapper().writeValueAsString(result));
    }

}
